<?php

declare(strict_types=1);

namespace Drupal\entity_extender\Plugin\EntityTypeExtender;

use Drupal\entity_extender\Plugin\EntityExtender\EntityLifecycleInterface;
use Drupal\entity_extender\Plugin\EntityExtender\EntityLifecycleTranslationInterface;

/**
 * The internal mapping of Drupal hooks and lifecycle events for extenders.
 *
 * @internal
 */
final class EntityHandlerStorageLifecycles {

  /**
   * Defines all possible lifecycle events.
   */
  public const LIFECYCLES = [
    EntityLifecycleInterface::class => [
      'create' => 'onCreate',
      'revision_create' => 'onCreateRevision',
      'revision_predelete' => 'onPreDeleteRevision',
      'revision_delete' => 'onPostDeleteRevision',
      'presave' => 'onPreSave',
      'insert' => 'onInsert',
      'update' => 'onUpdate',
      'predelete' => 'onPreDelete',
      'delete' => 'onPostDelete',
    ],
    EntityLifecycleTranslationInterface::class => [
      'translation_create' => 'onCreateTranslation',
      'translation_insert' => 'onInsertTranslation',
      'translation_delete' => 'onPostDeleteTranslation',
    ],
  ];

  /**
   * Checks the integrity of the {@see LIFECYCLES} definition.
   *
   * @return bool
   *   The return value is always `true` as the method is intended
   *   to be called within the `\assert`.
   *
   * @throws \ReflectionException
   */
  public static function checkIntegrity(): bool {
    $flat = \array_merge(...\array_values(self::LIFECYCLES));
    // Ensure there are no duplicates in subarrays.
    \assert(\count($flat) === \count(\array_unique($flat)));

    foreach (self::LIFECYCLES as $interface => $methods) {
      foreach ((new \ReflectionClass($interface))->getMethods() as $method) {
        // Ensure parameterless.
        \assert($method->getNumberOfRequiredParameters() === 0, $method->name);
        // Ensure defined in the mapping.
        \assert(\in_array($method->name, $methods, TRUE), $method->name);
      }
    }

    return TRUE;
  }

}

\assert(EntityHandlerStorageLifecycles::checkIntegrity());
